@@ -326,15 +326,16 @@ int loadfontfile(char *fontname)
 	 /* ISO-LatinX encodings where X=1 to 6 */
 
 	 if (strstr(tempname, "ISO") != NULL) {
-	    char estr[30];
+	    char estr[38];
 	    for (i = 0; i < 6; i++) {
 	       if (strchr(tempname, '1' + (char)i) != NULL) {
 	          flags |= ((i + 2) << 7);
 #ifdef TCL_WRAPPER
-		  sprintf(estr, "xcircuit::newencodingbutton ISOLatin%d", i + 1);
+		  snprintf(estr, sizeof(estr),
+			"xcircuit::newencodingbutton ISOLatin%d", i + 1);
 		  Tcl_Eval(xcinterp, estr);
 #else
-		  sprintf(estr, "ISO-Latin%d", i + 1);
+		  snprintf(estr, sizeof(estr), "ISO-Latin%d", i + 1);
 	          makenewencodingbutton(estr, (char)(i + 2));
 #endif
 		  break;
