@@ -92,7 +92,7 @@ void loadlgf(int mode)
    if (check_library(LGF_LIB) < 0) {
       int ilib;
       strcpy(_STR, LGF_LIB);
-      ilib = createlibrary();
+      ilib = createlibrary(FALSE);
       loadlibrary(ilib);
    }
    
