@@ -596,7 +596,7 @@ void UMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
    tmpb = yscale * sin(drot);
    tmpd = -scale * sin(drot);
    tmpe = yscale * cos(drot);
-   
+
    mata = ctm->a * tmpa + ctm->d * tmpb;
    matb = ctm->b * tmpa + ctm->e * tmpb;
    matc = ctm->c * tmpa + ctm->f * tmpb + position.x;
@@ -723,14 +723,15 @@ void UPreMultCTMbyMat(Matrix *ctm, Matrix *pre)
 void UTransformbyCTM(Matrix *ctm, XPoint *ipoints, XPoint *points, short number)
 {
    pointlist current, ptptr = points;
+   float fx, fy;
    short tmpx;
 
    for (current = ipoints; current < ipoints + number; current++, ptptr++) {
-      tmpx = (short)(ctm->a * (float)current->x + ctm->b * (float)current->y
-		 + ctm->c);
-      ptptr->y = (short)(ctm->d * (float)current->x + ctm->e * (float)current->y
-		 + ctm->f);
-      ptptr->x = tmpx;
+      fx = ctm->a * (float)current->x + ctm->b * (float)current->y + ctm->c;
+      fy = ctm->d * (float)current->x + ctm->e * (float)current->y + ctm->f;
+	
+      ptptr->x = (fx >= 0) ? (short)(fx + 0.5) : (short)(fx - 0.5);
+      ptptr->y = (fy >= 0) ? (short)(fy + 0.5) : (short)(fy - 0.5);
    }
 }
   
@@ -743,10 +744,13 @@ void UfTransformbyCTM(Matrix *ctm, XfPoint *fpoints, XPoint *points, short numbe
 {
    fpointlist current;
    pointlist new = points;
+   float fx, fy;
 
    for (current = fpoints; current < fpoints + number; current++, new++) {
-      new->x = (short)(ctm->a * current->x + ctm->b * current->y + ctm->c);
-      new->y = (short)(ctm->d * current->x + ctm->e * current->y + ctm->f);
+      fx = ctm->a * current->x + ctm->b * current->y + ctm->c;
+      fy = ctm->d * current->x + ctm->e * current->y + ctm->f;
+      new->x = (fx >= 0) ? (short)(fx + 0.5) : (short)(fx - 0.5);
+      new->y = (fy >= 0) ? (short)(fy + 0.5) : (short)(fy - 0.5);
    }
 }
 
@@ -1378,10 +1382,9 @@ void refresh(xcWidget bw, caddr_t clientdata, caddr_t calldata)
 
 void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   if (eventmode == NORMAL_MODE || eventmode == COPY2_MODE ||
-	 eventmode == PUSH_MODE || eventmode == PRESS_MODE ||
-	 eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
-	 eventmode == FONTCAT2_MODE) {
+   if (eventmode == NORMAL_MODE || eventmode == COPY_MODE ||
+	 eventmode == MOVE_MODE || eventmode == CATALOG_MODE ||
+	eventmode == FONTCAT_MODE || eventmode == EFONTCAT_MODE) {
 
       centerview(areastruct.topinstance);
       invalidate_graphics(topobject);
@@ -1703,55 +1706,62 @@ void UDrawBox(XPoint origin, XPoint corner)
 void UDrawRescaleBox(XPoint *corner)
 {
    XPoint 	origpoints[5], newpoints[5];
-   genericptr	*rgen = EDITPART;
-   float	savescale, newscale, f1, f2;
-   long		mindist, testdist;
+   genericptr	rgen;
+   float	savescale, newscale, lastscale, tempscale;
+   long		mindist, testdist, lastmindist;
    labelptr	rlab;
-   int		i, idx;
+   int		i, j, idx;
 
-   switch(ELEMENTTYPE(*rgen)) {
-      case LABEL:
-	 rlab = TOLABEL(rgen);
-	 labelbbox(rlab, origpoints, areastruct.topinstance);
-	 origpoints[5] = origpoints[0];
-	 savescale = rlab->scale;
-	 rlab->scale *= 2.0;
-	 labelbbox(rlab, newpoints, areastruct.topinstance);
-	 rlab->scale = savescale;
-	 newpoints[5] = newpoints[0];
-	 mindist = LONG_MAX;
-	 for (i = 0; i < 4; i++) {
-	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
-	    if (testdist < mindist) {
-	       mindist = testdist;
-	       idx = i;
-	    }
-	 }
-	 testdist = finddist(&origpoints[idx], &origpoints[idx + 1], corner);
-	 f2 = (float)sqrt((double)mindist);
-	 f1 = (float)sqrt((double)testdist);
-	 if (f1 == f2) break;
-	 newscale = (f2 - 2.0 * f1) / (f2 - f1);
-	 newscale *= rlab->scale;
-	 if (newscale > 20) newscale = 20;	/* temporary---set limits */
-	 if (newscale < 0.1) newscale = 0.1;
-	 rlab->scale = newscale;
-	 labelbbox(rlab, newpoints, areastruct.topinstance);
-	 rlab->scale = savescale;
-         break;
-      case GRAPHIC:
-         break;
-      case OBJINST:
-         break;
-      /* Note:  should be able to handle other elements, as well */
-   }
+   /* To be done---draw a box around each selected part, not	*/
+   /* just the first one.					*/
 
    SetFunction(dpy, areastruct.gc, GXxor);
    SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
    SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
    
-   UTransformbyCTM(DCTM, newpoints, origpoints, 4);
-   strokepath(origpoints, 4, 0, 1);
+   for (j = 0; j < areastruct.selects; j++) {
+      rgen = SELTOGENERIC(areastruct.selectlist + j);
+
+      switch(ELEMENTTYPE(rgen)) {
+         case LABEL:
+	    rlab = (labelptr)rgen;
+	    savescale = rlab->scale;
+	    newscale = lastscale = rlab->scale * 2;
+	    mindist = LONG_MAX;
+	    while (mindist > 4) {
+	       rlab->scale = newscale;
+	       labelbbox(rlab, newpoints, areastruct.topinstance);
+	       newpoints[5] = newpoints[0];
+	       lastmindist = mindist;
+	       mindist = LONG_MAX;
+	       for (i = 0; i < 4; i++) {
+	          testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	          if (testdist < mindist)
+	             mindist = testdist;
+	       }
+	       if (mindist == lastmindist)
+		  break;
+	       else if (lastmindist == LONG_MAX)
+		  newscale /= 2;
+	       else {
+	          tempscale = (fabs)((newscale * lastmindist) - (lastscale * mindist)) / 
+				(abs)(lastmindist - mindist);
+	          lastscale = newscale;
+	          newscale = tempscale;
+	       }
+	    }
+	    rlab->scale = savescale;
+            break;
+         case GRAPHIC:
+            break;
+         case OBJINST:
+            break;
+         /* Note:  should be able to handle other elements, as well */
+      }
+
+      UTransformbyCTM(DCTM, newpoints, origpoints, 4);
+      strokepath(origpoints, 4, 0, 1);
+   }
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1947,7 +1957,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    int		thispart;
-   XPoint 	bboxin, bboxout[2];
+   XPoint 	bboxin[2], bboxout[2];
    u_char	xm, ym;
    objectptr	theobject = theinstance->thisobject;
 
@@ -1964,10 +1974,13 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 
    /* do a quick test for intersection with the display window */
 
-   UTransformbyCTM(DCTM, &(theobject->bbox.lowerleft), &(bboxout[0]), 1);
-   bboxin.x = theobject->bbox.lowerleft.x + theobject->bbox.width;
-   bboxin.y = theobject->bbox.lowerleft.y + theobject->bbox.height; 
-   UTransformbyCTM(DCTM, &bboxin, &(bboxout[1]), 1);
+   bboxin[0].x = theobject->bbox.lowerleft.x;
+   bboxin[0].y = theobject->bbox.lowerleft.y;
+   bboxin[1].x = theobject->bbox.lowerleft.x + theobject->bbox.width;
+   bboxin[1].y = theobject->bbox.lowerleft.y + theobject->bbox.height; 
+   if (level == 0)
+      extendschembbox(theinstance, &(bboxin[0]), &(bboxin[1]));
+   UTransformbyCTM(DCTM, &bboxin, &bboxout, 2);
 
    xm = (bboxout[0].x < bboxout[1].x) ? 0 : 1;  
    ym = (bboxout[0].y < bboxout[1].y) ? 0 : 1;  
