@@ -124,7 +124,7 @@ void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
 /* Find the page object and instance with the indicated name.		*/
 /*----------------------------------------------------------------------*/
 
-objectptr NameToPageObject(char *objname, objinstptr *ret_inst)
+objectptr NameToPageObject(char *objname, objinstptr *ret_inst, int *ret_page)
 {
    int i;
 
@@ -132,6 +132,7 @@ objectptr NameToPageObject(char *objname, objinstptr *ret_inst)
       if (xobjs.pagelist[i]->pageinst == NULL) continue;
       if (!strcmp(objname, xobjs.pagelist[i]->pageinst->thisobject->name)) {
 	 if (ret_inst) *ret_inst = xobjs.pagelist[i]->pageinst;
+	 if (ret_page) *ret_page = i;
 	 return xobjs.pagelist[i]->pageinst->thisobject;
       }
    }
@@ -157,7 +158,7 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
    }
 
    if (dopages)
-      return NameToPageObject(objname, ret_inst);
+      return NameToPageObject(objname, ret_inst, NULL);
    else
       return NULL;
 }
@@ -230,10 +231,6 @@ void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (areastruct.selects > 0)
       connectivity(button, clientdata, calldata);
-   else if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 on wire to see connectivity.");
-      eventmode = CONNECT_MODE;
-   }
 }
 
 /*----------------------------------------------------------------------*/
@@ -287,8 +284,8 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
          ggen = *(nextselect->thisinst->thisobject->plist + *(nextselect->selectlist));
 	 while (rselect != NULL) {
 	    nextselect = rselect->next;
-	    free(rselect);
             free(rselect->selectlist);
+	    free(rselect);
 	    rselect = nextselect;
 	 }
       }
@@ -670,6 +667,7 @@ void collectsubschems(int toppage)
    findsubschems(loctop, cschem, 0, pagelist);
 
    for (pageno = 0; pageno < xobjs.pages; pageno++) {
+      if (pageno == loctop) continue;
       if (pagelist[pageno] > 0) {
 	  if (xobjs.pagelist[pageno]->filename != NULL)
 	     free(xobjs.pagelist[pageno]->filename);
@@ -680,49 +678,6 @@ void collectsubschems(int toppage)
    free((char *)pagelist);
 }
 
-/*--------------------------------------------------------*/
-/* Copy label to corresponding schematic/symbol		  */
-/*--------------------------------------------------------*/
-
-void copypinlabel(labelptr pinlab)
-{
-   labelptr *newlabel, tlabel;
-   genericptr *tgen;
-   objectptr schemobj = topobject->symschem;
-
-   if (areastruct.schemon && schemobj != NULL && (pinlab->pin == LOCAL)) {
-
-      /* If this pin already exists, don't copy it */
-
-      for (tgen = schemobj->plist; tgen < schemobj->plist + schemobj->parts;
-		tgen++) {
-         if (IS_LABEL(*tgen)) {
-	    tlabel = TOLABEL(tgen);
-	    if (!stringcomp(tlabel->string, pinlab->string)) return;
-         }
-      }
-
-      NEW_LABEL(newlabel, schemobj);
-      (*newlabel)->pin = pinlab->pin;
-      (*newlabel)->rotation = pinlab->rotation;
-      (*newlabel)->justify = pinlab->justify;
-      (*newlabel)->color = pinlab->color;
-      (*newlabel)->scale = pinlab->scale;
-      (*newlabel)->string = stringcopy(pinlab->string);
-      (*newlabel)->passed = NULL;
-
-      /* place label just outside bounding box, then recompute bbox */
-
-      (*newlabel)->position.x = schemobj->bbox.lowerleft.x + (schemobj->bbox.width >> 1);
-      (*newlabel)->position.y = schemobj->bbox.lowerleft.y - TEXTHEIGHT
-	   * (*newlabel)->scale;
-      schemobj->parts++;
-      incr_changes(schemobj);
-      invalidate_netlist(schemobj);
-      singlebbox(ENDPART);
-   }
-}
-
 /*-------------------------------------------------------*/
 /* Check if top-level page is the same name as a library */
 /* object; if so, connect it.				 */
@@ -785,8 +740,7 @@ int checksym(objectptr symobj, char *cname)
 
 /*----------------------------------------------------------------------*/
 /* Find location of corresponding pin in symbol/schematic and change it */
-/* to the text of the indicated label.  If "newlabel" is NULL, then	*/
-/* change the other label from pin to normal text.			*/
+/* to the text of the indicated label.					*/
 /*----------------------------------------------------------------------*/
 
 void changeotherpins(labelptr newlabel, stringpart *oldstring)
@@ -800,40 +754,51 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
       if (IS_LABEL(*tgen)) {
 	 tlab = TOLABEL(tgen);
+	 if (tlab->pin != LOCAL) continue;
 	 if (!stringcomp(tlab->string, oldstring)) {
 	    if (newlabel != NULL) {
 	       free(tlab->string);
 	       tlab->string = stringcopy(newlabel->string);
 	    }
-	    else
-	       tlab->pin = NORMAL;
 	 }
       }
    }
 }
 
 /*----------------------------------------------------------------------*/
-/* Swap object schematic and symbol pages.   				*/
-/*  mode = 0 disallows creation of a new schematic or symbol; i.e., if	*/
-/*  there is no corresponding schematic/symbol, nothing happens.	*/
+/* Xt wrapper for swapschem()						*/
 /*----------------------------------------------------------------------*/
 
-void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
+void xlib_swapschem(xcWidget w, pointertype mode, caddr_t calldata)
+{
+   swapschem((int)mode, -1);
+}
+
+/*----------------------------------------------------------------------*/
+/* Swap object schematic and symbol pages.   				*/
+/*  allow_create = 0 disallows creation of a new schematic or symbol;	*/
+/*  i.e., if there is no corresponding schematic/symbol, nothing	*/
+/*  happens.								*/
+/*----------------------------------------------------------------------*/
+
+void swapschem(int allow_create, int libnum)
 {
    objectptr savepage = topobject;
    labelptr  *pinlab;
    genericptr *plab;
    Boolean lflag;
    pushlistptr stacktop;
+   short loclibnum = libnum;
 
    if (areastruct.schemon == False) return;
+   if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
-   if (eventmode == PRESS_MODE || eventmode == COPY2_MODE)
+   if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
       standard_element_delete(NORMAL);
 
-   /* Create symbol or schematic, if allowed by mode */
+   /* Create symbol or schematic, if allowed by allow_create */
 
-   if ((topobject->symschem == NULL) && (mode != 0)
+   if ((topobject->symschem == NULL) && (allow_create != 0)
 		&& (topobject->schemtype != SECONDARY)) {
       if (topobject->schemtype != PRIMARY) {
 	 int tpage;
@@ -854,15 +819,14 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
       }
       else {
 	 objectptr *newobject;
-	 short libnum = USERLIB - LIBRARY;
 
 	 /* create a new library object for the new symbol */
 
-	 xobjs.userlibs[libnum].library = (objectptr *)
-		realloc(xobjs.userlibs[libnum].library,
-		++xobjs.userlibs[libnum].number * sizeof(objectptr));
-	 newobject = xobjs.userlibs[libnum].library
-		+ xobjs.userlibs[libnum].number - 1;
+	 xobjs.userlibs[loclibnum].library = (objectptr *)
+		realloc(xobjs.userlibs[loclibnum].library,
+		++xobjs.userlibs[loclibnum].number * sizeof(objectptr));
+	 newobject = xobjs.userlibs[loclibnum].library
+		+ xobjs.userlibs[loclibnum].number - 1;
          *newobject = (objectptr) malloc(sizeof(object));
 	 initmem(*newobject);
 	 (*newobject)->schemtype = SYMBOL;
@@ -874,7 +838,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 	 /* top instance to point to it.			    */
 
 	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = addtoinstlist(libnum, *newobject, FALSE);
+	 areastruct.topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
 	 calcbbox(areastruct.topinstance);
@@ -930,7 +894,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
       calcbbox(areastruct.topinstance);
 
       /* Recreate the user library with the new symbol */
-      if (savepage->schemtype != SYMBOL) composelib(USERLIB);
+      if (savepage->schemtype != SYMBOL) composelib(loclibnum + LIBRARY);
    }
    else if (topobject->symschem != NULL) {
 
@@ -991,6 +955,8 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
    setsymschem();
 }
 
+#ifndef TCL_WRAPPER
+
 /*----------------------------------------------------------------------*/
 /* Wrapper for swapschem() when generating a new symbol.		*/
 /*----------------------------------------------------------------------*/
@@ -1001,7 +967,7 @@ void makesymbol(xcWidget w, caddr_t calldata)
 
    strcpy(topobject->name, _STR2);
    checkname(topobject);
-   swapschem(w, (pointertype)1, calldata);
+   swapschem(1, -1);
 }
 
 /*----------------------------------------------------------------------*/
@@ -1030,9 +996,11 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
       popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, NULL);
    }
    else
-      swapschem(w, (pointertype)1, calldata);
+      swapschem(1, -1);
 }
 
+#endif /* !TCL_WRAPPER */
+
 /*------------------------------------------*/
 /* Disassociate a symbol from its schematic */
 /*------------------------------------------*/
@@ -1073,12 +1041,12 @@ void startschemassoc(xcWidget w, pointertype mode, caddr_t calldata)
       if (topobject->schemtype == PRIMARY) {
 	 /* Find a symbol to associate */
 	 startcatalog(w, LIBLIB, NULL);
-	 Wprintf("Click on library page, then symbol to associate.");
+	 Wprintf("Select library page, then symbol to associate.");
       }
       else {
 	 /* Find a schematic (page) to associate */
 	 startcatalog(w, PAGELIB, NULL);
-	 Wprintf("Click on schematic page to associate.");
+	 Wprintf("Select schematic page to associate.");
       }
    }
 }
