@@ -33,12 +33,12 @@
 /*----------------------------------------------------------------------*/
 #include "prototypes.h"
 
-/*-------------------------------------------------------------------------*/
-/* Global Variable definitions						   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Exported Variable definitions					*/
+/*----------------------------------------------------------------------*/
 
 extern short eventmode;	/* keep track of the mode the screen is in */
-extern Display	*dpy;   /* Works well to make this globally accessible */
+extern Display	*dpy;
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
 extern Clientdata areastruct;
@@ -101,7 +101,7 @@ void reviseselect(short *slist, int selects, short *removed)
 /* Draw a selected item */
 /*----------------------*/
 
-void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curinst)
+void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 {
    genericptr elementptr = *(curobj->plist + instance);
 
@@ -124,6 +124,9 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
       case OBJINST:
          UDrawObject((objinstptr)elementptr, SINGLE, mode, NULL);
          break;
+      case GRAPHIC:
+         UDrawGraphic((graphicptr)elementptr);
+	 break;
    }
 }
 
@@ -133,16 +136,16 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
 
 void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 {
-   XSetFunction(dpy, areastruct.gc, GXcopy);
-   XSetForeground(dpy, areastruct.gc, BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXcopy);
+   SetForeground(dpy, areastruct.gc, BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
    indicateparams(*(curobj->plist + *newselect));
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
 }
 
@@ -358,26 +361,17 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
       /* when selection element to attach to, don't select the element */
       /* to be attached. . .					       */
 
-      if (attachto == 1 && eventmode != COPY2_MODE && eventmode !=
-	    PRESS_MODE && curgen == EDITPART) continue;
+      if (attachto == 1 && eventmode != COPY_MODE && eventmode !=
+	    MOVE_MODE && curgen == EDITPART) continue;
 
       selected = False;
 
-      /* check for position inside the bounding box of one of the objects */
+      /* Check among polygons, arcs, and curves */
 
-      if ((*curgen)->type == (class & OBJINST)) {
-
-	 objinstbbox(TOOBJINST(curgen), newboxpts);
-
-         /* Look for an intersect of the boundingbox and pointer position. */
-         /* This is a simple matter of rotating the pointer position with  */
-         /*  respect to the origin of the bounding box segment, as if the  */
-         /*  segment were rotated to 0 degrees.  The sign of the resulting */
-         /*  point's y-position is the same for all bbox segments if the   */
-         /*  pointer position is inside the bounding box.		   */
-
-	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
-		newboxpts);
+      if (((*curgen)->type == (class & POLYGON)) ||
+		((*curgen)->type == (class & ARC)) ||
+		((*curgen)->type == (class & SPLINE))) {
+	  selected = pathselect(curgen, class, range);
       }
 
       else if ((*curgen)->type == (class & LABEL)) {
@@ -391,13 +385,32 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 
 	 labelbbox(curlab, newboxpts, selinst);
 
-         /* check for point inside bounding box, as for objects */
+	 /* Need to check for zero-size boxes or test_insideness()	*/
+	 /* fails.  Zero-size boxes happen when labels are parameters	*/
+	 /* set to a null string.					*/
 
-	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+	 if ((newboxpts[0].x != newboxpts[1].x) || (newboxpts[0].y !=
+		newboxpts[1].y)) {
+	
+            /* check for point inside bounding box, as for objects */
+
+	    selected = test_insideness(areastruct.save.x, areastruct.save.y,
 		newboxpts);
-	 if (selected) textpos = textend = 0;
+	    if (selected) textpos = textend = 0;
+	 }
       }
 
+      else if ((*curgen)->type == (class & GRAPHIC)) {
+
+         /* Look among the graphic images */
+
+	 graphicptr curg = TOGRAPHIC(curgen);
+	 graphicbbox(curg, newboxpts);
+
+         /* check for point inside bounding box, as for objects */
+	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+		newboxpts);
+      }
 
       else if ((*curgen)->type == (class & PATH)) {
 
@@ -417,9 +430,20 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 	    }
       }
 
-      /* Check among polygons, arcs, and curves */
+      else if ((*curgen)->type == (class & OBJINST)) {
 
-      else selected = pathselect(curgen, class, range);
+	 objinstbbox(TOOBJINST(curgen), newboxpts, True);
+
+         /* Look for an intersect of the boundingbox and pointer position. */
+         /* This is a simple matter of rotating the pointer position with  */
+         /*  respect to the origin of the bounding box segment, as if the  */
+         /*  segment were rotated to 0 degrees.  The sign of the resulting */
+         /*  point's y-position is the same for all bbox segments if the   */
+         /*  pointer position is inside the bounding box.		   */
+
+	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+		newboxpts);
+      }
 
       /* Add this object to the list of things found under the cursor */
 
@@ -530,13 +554,21 @@ void selectarea()
 
       switch(ELEMENTTYPE(*curgen)) {
 	case(OBJINST):
-           /* check for instance of object inside area box */
+           /* check for instance of object center point inside area box */
            selected = (TOOBJINST(curgen)->position.x < areastruct.save.x) &&
 		(TOOBJINST(curgen)->position.x > areastruct.origin.x) &&
 		(TOOBJINST(curgen)->position.y < areastruct.save.y) &&
 		(TOOBJINST(curgen)->position.y > areastruct.origin.y);
 	   break;
 
+	case(GRAPHIC):
+           /* check for graphic image center point inside area box */
+           selected = (TOGRAPHIC(curgen)->position.x < areastruct.save.x) &&
+		(TOGRAPHIC(curgen)->position.x > areastruct.origin.x) &&
+		(TOGRAPHIC(curgen)->position.y < areastruct.save.y) &&
+		(TOGRAPHIC(curgen)->position.y > areastruct.origin.y);
+	   break;
+
         case(LABEL): {
 	   XPoint bboxpts[4], newboxpts[4], adj;
 	   labelptr slab = TOLABEL(curgen);
@@ -665,10 +697,6 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
 	 Wprintf("Nothing to deselect!");
       else if (areastruct.selects == 1)
 	 unselect_all();
-      else {
-         eventmode = DESEL_MODE;
-         Wprintf("Click on element to deselect");
-      }
    }
 }
 
@@ -680,6 +708,7 @@ void draw_all_selected()
 {
    int j;
 
+   if (areastruct.hierstack != NULL) return;
    for (j = 0; j < areastruct.selects; j++)
       gendrawselected(areastruct.selectlist + j, topobject, areastruct.topinstance);
 }
@@ -693,10 +722,11 @@ void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
    short *lastselect;
 
    if (areastruct.selects == 0) return;
+   else if (areastruct.hierstack != NULL) return;
 
    /* reset the graphics state */
 
-   XSetFunction(dpy, areastruct.gc, GXcopy);
+   SetFunction(dpy, areastruct.gc, GXcopy);
 
    for (lastselect = areastruct.selectlist; lastselect < areastruct.selectlist 
 	+ areastruct.selects; lastselect++) {
@@ -705,6 +735,23 @@ void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
    }
 }
 
+/*----------------------------------------------------------------------*/
+/* Free a selection linked-list structure				*/
+/* (don't confuse with freeselects)					*/
+/*----------------------------------------------------------------------*/
+
+static void freeselection(selection *rselect)
+{
+   selection *nextselect;
+
+   while (rselect != NULL) {
+      nextselect = rselect->next;
+      free(rselect->selectlist);
+      free(rselect);
+      rselect = nextselect;
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Free memory from the previous selection list, copy the	*/
 /* current selection list to the previous selection list, and	*/
@@ -719,6 +766,7 @@ void freeselects()
    if (areastruct.selects > 0)
       free(areastruct.selectlist);
    areastruct.selects = 0;
+   free_stack(&areastruct.hierstack);
 }
 
 /*--------------------------------------------------------------*/
@@ -750,7 +798,7 @@ void clearselects()
 }
 
 /*--------------------------------------------------------------*/
-/* Unselect all the selected objects and free memory from the	*/
+/* Unselect all the selected elements and free memory from the	*/
 /* selection list.						*/
 /*--------------------------------------------------------------*/
 
@@ -760,7 +808,6 @@ void unselect_all()
    clearselects();
 }
 
-
 /*----------------------------------------------------------------------*/
 /* Select the nearest element, searching the hierarchy if necessary.	*/
 /* Return an pushlist pointer to a linked list containing the 		*/
@@ -772,7 +819,7 @@ void unselect_all()
 
 selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 {
-   selection *rselect, *rcheck, *lastselect, *nextselect;
+   selection *rselect, *rcheck, *lastselect;
    genericptr rgen;
    short i, *selectobj, numselect, origselect;
    short rclass;
@@ -861,12 +908,7 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
    }
    rselect->selects -= unselects;
    if (rselect->selects == 0) {
-      while (rselect != NULL) {
-         nextselect = rselect->next;
-         free(rselect);
-         free(rselect->selectlist);
-         rselect = nextselect;
-      }
+      freeselection(rselect);
       rselect = NULL;
    }
    return rselect;
@@ -876,34 +918,23 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 /* Start drawing a select area box. */
 /*----------------------------------*/
 
-#ifdef TCL_WRAPPER
-
-xcTimeOutProc startselect(caddr_t clientdata)
+void startselect()
 {
-   if (eventmode != PENDING_MODE) return;
    eventmode = SELAREA_MODE;
    areastruct.origin.x = areastruct.save.x;
    areastruct.origin.y = areastruct.save.y;
    UDrawBox(areastruct.origin, areastruct.save);
 
+#ifdef TCL_WRAPPER
    Tk_CreateEventHandler(areastruct.area, ButtonMotionMask,
 		(Tk_EventProc *)xctk_drag, NULL);
-}
-
 #else
-
-xcTimeOutProc startselect(caddr_t clientdata, xcIntervalId *id)
-
-{
-   if (eventmode != PENDING_MODE) return;
-   eventmode = SELAREA_MODE;
-   areastruct.origin.x = areastruct.save.x;
-   areastruct.origin.y = areastruct.save.y;
-   UDrawBox(areastruct.origin, areastruct.save);
-}
-
+   xcAddEventHandler(areastruct.area, ButtonMotionMask, False,
+		(xcEventHandler)xlib_drag, NULL);
 #endif
 
+}
+
 /*-------------------------*/
 /* Track a select area box */
 /*-------------------------*/
@@ -923,16 +954,89 @@ void trackselarea()
    areastruct.save.y = newpos.y;
 }
 
+/*----------------------*/
+/* Track a rescale box	*/
+/*----------------------*/
+
+void trackrescale()
+{
+   XPoint newpos;
+   u_int  nullui;
+
+   newpos = UGetCursorPos();
+   if (newpos.x == areastruct.save.x && newpos.y == areastruct.save.y) return;
+
+   UDrawRescaleBox(&areastruct.save);
+   UDrawRescaleBox(&newpos);
+
+   areastruct.save.x = newpos.x;
+   areastruct.save.y = newpos.y;
+}
+
+/*----------------------------------------------------------------------*/
+/* Polygon distance comparison function for qsort			*/
+/*----------------------------------------------------------------------*/
+
+int dcompare(const void *a, const void *b)
+{
+   XPoint cpt;
+   genericptr agen, bgen;
+   short j, k, adist, bdist;
+
+   cpt.x = areastruct.save.x;
+   cpt.y = areastruct.save.y;
+
+   j = *((short *)a);
+   k = *((short *)b);
+
+   agen = *(topobject->plist + j);
+   bgen = *(topobject->plist + k);
+
+   if (agen->type != POLYGON || bgen->type != POLYGON) return 0;
+
+   adist = closedistance((polyptr)agen, &cpt);
+   bdist = closedistance((polyptr)bgen, &cpt);
+
+   if (adist == bdist) return 0;
+   return (adist < bdist) ? 1 : -1;
+}
+
+/*----------------------------------------------------------------------*/
+/* Compare two selection linked lists					*/
+/*----------------------------------------------------------------------*/
+
+Boolean compareselection(selection *sa, selection *sb)
+{
+   int i, j, match;
+   short n1, n2;
+
+   if ((sa == NULL) || (sb == NULL)) return False;
+   if (sa->selects != sb->selects) return False;
+   match = 0;
+   for (i = 0; i < sa->selects; i++) {
+      n1 = *(sa->selectlist + i);
+      for (j = 0; j < sb->selects; j++) {
+         n2 = *(sb->selectlist + j);
+	 if (n1 == n2) {
+	    match++;
+	    break;
+	 }
+      }
+   }
+   return (match == sa->selects) ? True : False;
+}
+
 /*----------------------------------------------------------------------*/
 /* Recursive selection mechanism					*/
 /*----------------------------------------------------------------------*/
 
 short *recurse_select_element(short class, u_char mode) {
    pushlistptr seltop, nextptr;
-   selection *rselect, *nextselect;
-   short *newselect, *desel;
-   int i, j, k, ilast;
-   static int pick = 0;
+   selection *rselect;
+   short *newselect, *desel, localpick;
+   static short pick = 0;
+   static selection *saveselect = NULL;
+   int i, j, k, ilast, jlast;
    Boolean unselect = False;
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
@@ -947,29 +1051,54 @@ short *recurse_select_element(short class, u_char mode) {
    }
    rselect = recurselect(class, mode, &seltop);
 
+   if (rselect) {
+      /* Order polygons according to nearest point distance. */
+      qsort((void *)rselect->selectlist, (size_t)rselect->selects,
+		sizeof(short), dcompare);
+
+      if (compareselection(rselect, saveselect))
+	 pick++;
+      else
+	 pick = 0;
+
+      localpick = pick % rselect->selects;
+   }
+
    /* Mechanism for unselecting elements under the cursor	*/
    /* (Unselect all picked objects)				*/
 
    if (rselect && unselect) {
+
+      ilast = -1;
+      k = 0;
       for (i = 0; i < rselect->selects; i++) {
-	 newselect = rselect->selectlist + i;
-         for (j = 0; j < areastruct.selects; j++) {
-            if (*(areastruct.selectlist + j) == *newselect) {
-               XSetFunction(dpy, areastruct.gc, GXcopy);
-               XTopSetForeground(GSELTOCOLOR(topobject, newselect));
-               geneasydraw(*newselect, DEFAULTCOLOR, topobject, areastruct.topinstance);
-               areastruct.selects--;
-               for (k = j; k < areastruct.selects; k++)
-	          *(areastruct.selectlist + k) = *(areastruct.selectlist + k + 1);
-	       break;
+	 for (j = 0; j < areastruct.selects; j++) {
+	    if (*(areastruct.selectlist + j) == *(rselect->selectlist + i)) {
+	       jlast = j;
+	       ilast = i;
+	       if (++k == localpick)
+	          break;
 	    }
 	 }
+	 if (j < areastruct.selects) break;
       }
-      if (areastruct.selects == 0) {
-	 free (areastruct.selectlist);
-	 areastruct.selectlist = NULL;
+      if (ilast >= 0) {
+	 newselect = rselect->selectlist + ilast;
+         SetFunction(dpy, areastruct.gc, GXcopy);
+         XTopSetForeground(SELTOCOLOR(newselect));
+         geneasydraw(*newselect, DEFAULTCOLOR, topobject, areastruct.topinstance);
+         areastruct.selects--;
+         for (k = jlast; k < areastruct.selects; k++)
+	    *(areastruct.selectlist + k) = *(areastruct.selectlist + k + 1);
+
+         if (areastruct.selects == 0) freeselects();
+
+         /* Register the selection as an undo event */
+         register_for_undo(XCF_Select, mode, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
       }
    }
+
    else if (rselect) {
 
       /* Mechanism for selecting objects:			*/
@@ -986,7 +1115,7 @@ short *recurse_select_element(short class, u_char mode) {
 	 }
 	 if (j == areastruct.selects) {
 	    ilast = i;
-	    if (++k == pick)
+	    if (++k == localpick)
 	       break;
 	 }
       }
@@ -998,12 +1127,11 @@ short *recurse_select_element(short class, u_char mode) {
          setoptionmenu();
          u2u_snap(&areastruct.save);
 
-         /* Register the selection as an undo event */
+         /* Register the selection as an undo event	*/
+	 /* (only if selection changed)			*/
+
          register_for_undo(XCF_Select, mode, areastruct.topinstance,
 		areastruct.selectlist, areastruct.selects);
-
-         /* Make sure this object is not the first to be picked next time */
-         if (++pick >= rselect->selects) pick = 0;
       }
    }
 
@@ -1014,13 +1142,9 @@ short *recurse_select_element(short class, u_char mode) {
       free(seltop);
       seltop = nextptr;
    }
-   while (rselect != NULL) {
-      nextselect = rselect->next;
-      free(rselect);
-      free(rselect->selectlist);
-      rselect = nextselect;
-   }
+
+   freeselection(saveselect);
+   saveselect = rselect;
 
    return areastruct.selectlist;
 }
-
