@@ -450,6 +450,34 @@ int findcurfont(int tpos, stringpart *strtop, objinstptr thisinst)
    return cfont;
 }
 
+/*----------------------------------------------------------------------*/
+/* Return a local position and stringpart for the first occurrence of	*/
+/* "substring" in the the indicated xcircuit string.  If non-NULL,	*/
+/* "locpos" is set to the position of the substring in the stringpart,	*/
+/* or -1 if the text was not found.  Text cannot cross stringpart	*/
+/* boundaries (although this should be allowed).			*/
+/*----------------------------------------------------------------------*/
+
+stringpart *findtextinstring(char *search, int *locpos, stringpart *strtop,
+	objinstptr localinst)
+{
+   stringpart *strptr = strtop;
+   char *strstart;
+
+   for (strptr = strtop; strptr != NULL; strptr = nextstringpart(strptr, localinst)) {
+      if ((strptr->type == TEXT_STRING) && strptr->data.string) {
+	 strstart = strstr(strptr->data.string, search);
+	 if (strstart != NULL) {
+	    if (locpos != NULL)
+	       *locpos = (int)(strstart - (char *)strptr->data.string);
+	    return strptr;
+	 }
+      }
+   }
+   if (locpos != NULL) *locpos = -1;
+   return NULL;
+}
+
 /*----------------------------------------------------------------------*/
 /* Return a local position and stringpart for "tpos" positions into	*/
 /* the indicated string.  Position and stringpart are for the character */
@@ -676,9 +704,11 @@ int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localins
    char *tptr = text;
    char *sptr;
    int rval, llen = strlen(text), slen;
+   Boolean has_text = FALSE;
 
    for (strptr = string; strptr != NULL; strptr = nextstringpart(strptr, localinst)) {
       if (strptr->type == TEXT_STRING) {
+	 has_text = TRUE;
 	 sptr = strptr->data.string;
 	 slen = min(strlen(sptr), llen);
 	 llen -= slen;
@@ -692,7 +722,9 @@ int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localins
 	    tptr += slen;
       }
    }
-   return 0;
+
+   /* Check condition that no text was encountered in the xcircuit string */
+   return ((llen > 0) && !has_text) ? 1 : 0;
 }
 
 /*----------------------------------------------------------------------*/
@@ -1504,7 +1536,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 		   group = 0;
 		   for (textptr = strptr->nextpart->data.string;
 				textptr && *textptr != '\0'; textptr++) {
-		      charptr = fonts[ffont].encoding[*(unsigned char *)textptr];
+		      charptr = fonts[ffont].encoding[*(u_char *)textptr];
 		      tmpheight = (int)((float)charptr->bbox.height
 				* fonts[ffont].scale);
 		      if (group < tmpheight) group = tmpheight;
@@ -1888,14 +1920,14 @@ void composefontlib(short cfont)
 /* ButtonPress handler during font catalog viewing mode */
 /*------------------------------------------------------*/
 
-void fontcatbutton(XButtonEvent *event)
+void fontcat_op(int op, int x, int y)
 {
    short chx, chy;
    u_long rch = 0;
 
-   if (event->button == Button1 || event->button == Button2) {
+   if (op != XCF_Cancel) {
 
-      window_to_user(event->x, event->y, &areastruct.save);
+      window_to_user(x, y, &areastruct.save);
  
       chy = -areastruct.save.y / del + 1;
       chx = areastruct.save.x / del;
