@@ -129,15 +129,12 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 /* Names used for convenience throughout the source code */
 
 #define topobject	areastruct.topinstance->thisobject
+#define hierobject	areastruct.hierstack->thisinst->thisobject
 
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   ((areastruct.drawstack != NULL) ? \
-			OBJINST : \
-			((*(topobject->plist + *(a)))->type & ALL_TYPES))
-#define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
-#define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
+
 #define ENDPART		topobject->plist + topobject->parts
 #define EDITPART	topobject->plist + areastruct.editpart
 
@@ -164,21 +161,26 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define TOSPLINE(a)	(*((splineptr *)(a))) 
 #define TOPATH(a)	(*((pathptr *)(a)))
 #define TOGRAPHIC(a)	(*((graphicptr *)(a)))
+#define TOGENERIC(a)	(*((genericptr *)(a)))
 
 /* conversions from a selection to a specific type */
 
-#define SELTOPOLY(a)	TOPOLY(topobject->plist + (*(a)))
-#define SELTOLABEL(a)	TOLABEL(topobject->plist + (*(a)))
-#define SELTOOBJINST(a)	((areastruct.drawstack != NULL) ? \
-			areastruct.drawstack->thisinst : \
-			TOOBJINST(topobject->plist + (*(a))))
-#define SELTOARC(a)	TOARC(topobject->plist + (*(a)))
-#define SELTOSPLINE(a)	TOSPLINE(topobject->plist + (*(a)))
-#define SELTOPATH(a)	TOPATH(topobject->plist + (*(a)))
-#define SELTOGRAPHIC(a)	TOGRAPHIC(topobject->plist + (*(a)))
-#define SELTOGENERIC(a) ((areastruct.drawstack != NULL) ? \
-			(genericptr)areastruct.drawstack->thisinst : \
-			(*(topobject->plist + (*(a)))))
+#define SELTOGENERICPTR(a) ((areastruct.hierstack == NULL) ? \
+			(topobject->plist + *(a)) : \
+			(hierobject->plist + *(a)))
+
+#define SELTOPOLY(a)	TOPOLY(SELTOGENERICPTR(a))
+#define SELTOLABEL(a)	TOLABEL(SELTOGENERICPTR(a))
+#define SELTOOBJINST(a)	TOOBJINST(SELTOGENERICPTR(a))
+#define SELTOARC(a)	TOARC(SELTOGENERICPTR(a))
+#define SELTOSPLINE(a)	TOSPLINE(SELTOGENERICPTR(a))
+#define SELTOPATH(a)	TOPATH(SELTOGENERICPTR(a))
+#define SELTOGRAPHIC(a)	TOGRAPHIC(SELTOGENERICPTR(a))
+#define SELTOGENERIC(a)	TOGENERIC(SELTOGENERICPTR(a))
+
+#define SELECTTYPE(a)   ((SELTOGENERIC(a))->type & ALL_TYPES)
+#define SELTOCOLOR(a)	((SELTOGENERIC(a))->color)
+
 
 /* creation of new elements */
 
@@ -271,8 +273,8 @@ typedef struct {
 
 /*----------------------------------------------------------------------*/
 
-#define RADFAC 0.0174532925199   /* pi / 180 */
-#define INVRFAC 57.295779       /* 180 / pi */
+#define RADFAC 0.0174532925199  /* (pi / 180) */
+#define INVRFAC 57.295779 /* (180 / pi) */
 
 /*----------------------------------------------------------------------*/
 
@@ -284,14 +286,30 @@ typedef struct {
 /* Event mode definitions (state of drawing area)			*/
 /*----------------------------------------------------------------------*/
 
-enum editmode {NORMAL_MODE = 0, UNDO_MODE,
-PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
-PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
-CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
-TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, POLY_MODE, BOX0_MODE,
-BOX_MODE, EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE,
-SPLINE_MODE, ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE,
-ULPARAM_MODE, UIPARAM_MODE, CONNECT_MODE, ASSOC_MODE};
+enum editmode {
+  NORMAL_MODE = 0,	/* On the drawing page, none of the situations below */
+  UNDO_MODE,		/* In the process of an undo/redo operation */
+  MOVE_MODE,		/* In the process of moving elements */
+  COPY_MODE,		/* In the process of copying elements */
+  PAN_MODE,		/* In the process of panning to follow the cursor */
+  SELAREA_MODE,		/* Area selection box */
+  RESCALE_MODE,		/* Interactive element rescaling box */
+  CATALOG_MODE,		/* On a library page, library directory, or page directory */
+  CATTEXT_MODE,		/* Editing an existing object name in the library */
+  FONTCAT_MODE,		/* Accessing the font character page from TEXT2_MODE */
+  EFONTCAT_MODE,	/* Accessing the font character page from TEXT3_MODE */
+  TEXT_MODE,		/* Creating a new label */
+  WIRE_MODE,		/* Creating a new polygon (wire) */
+  BOX_MODE,		/* Creating a new box */
+  ARC_MODE,		/* Creating a new arc */
+  SPLINE_MODE,		/* Creating a new spline */
+  ETEXT_MODE,		/* Editing an exiting label */
+  EPOLY_MODE,		/* Editing an existing polygon */
+  EARC_MODE,		/* Editing an existing arc */
+  ESPLINE_MODE,		/* Editing an existing spline */
+  EPATH_MODE,		/* Editing an existing path */
+  ASSOC_MODE		/* Choosing an associated schematic or symbol */
+};
 
 /*----------------------------------------------------------------------*/
 /* File loading modes							*/
@@ -362,6 +380,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define DEC_INCH	0
 #define FRAC_INCH	1
 #define CM		2
+#define INTERNAL	3
 
 /*----------------------------------------------------------------------*/
 /* Library types							*/
@@ -462,7 +481,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Cursor definitions 							*/
 /*----------------------------------------------------------------------*/
 
-#define NUM_CURSORS	10
+#define NUM_CURSORS	11
 
 #define ARROW 		appcursors[0]
 #define CROSS 		appcursors[1]
@@ -474,6 +493,9 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define CIRCLE		appcursors[7]
 #define QUESTION	appcursors[8]
 #define WAITFOR		appcursors[9]
+#define HAND		appcursors[10]
+
+#define DEFAULTCURSOR	(*areastruct.defaultcursor)
 
 /*----------------------------------------------------------------------*/
 /* selection-mechanism structures					*/
@@ -609,7 +631,8 @@ typedef struct {
 typedef struct _objlist *objlistptr;
 
 typedef struct _objlist {
-   objectptr	thisobject;
+   int		libno;		/* library in which object appears */
+   objectptr	thisobject;	/* pointer to the object */
    objlistptr	next;
 } objlist;
 
@@ -987,6 +1010,9 @@ typedef struct _alias {
 #define xcRemoveTimeOut			XtRemoveTimeOut
 #define xcTimeOutProc			XtTimerCallbackProc
 #define xcIntervalId			XtIntervalId
+#ifndef ClientData
+   #define ClientData			XtPointer
+#endif
 
 #endif
 
@@ -1133,6 +1159,8 @@ typedef struct _keybinding {
 
 /*----------------------------------------------------------------------*/
 /* Enumeration of functions available for binding to keys/buttons	*/
+/* IMPORTANT!  Do not alter this list without also fixing the text	*/
+/* in keybindings.c!							*/
 /*----------------------------------------------------------------------*/
 
 enum {
@@ -1165,31 +1193,33 @@ enum {
    XCF_View /* 50 */,		XCF_Zoom_In /* 51 */,
    XCF_Zoom_Out /* 52 */,	XCF_Pan /* 53 */,
    XCF_Double_Snap /* 54 */,	XCF_Halve_Snap /* 55 */,
-   XCF_Pan_Left /* 56 */,	XCF_Pan_Right /* 57 */,
-   XCF_Pan_Up /* 58 */,		XCF_Pan_Down /* 59 */,
-   XCF_Write /* 60 */,		XCF_Rotate /* 61 */,
-   XCF_Flip_X /* 62 */,		XCF_Flip_Y /* 63 */,
-   XCF_Snap /* 64 */,		XCF_SnapTo /* 65 */,
-   XCF_Pop /* 66 */,		XCF_Push /* 67 */,
-   XCF_Delete /* 68 */,		XCF_Select /* 69 */,
-   XCF_Box /* 70 */,		XCF_Arc /* 71 */,
-   XCF_Text /* 72 */,		XCF_Exchange /* 73 */,
-   XCF_Copy /* 74 */,		XCF_Move /* 75 */,
-   XCF_Join /* 76 */,		XCF_Unjoin /* 77 */,
-   XCF_Spline /* 78 */,		XCF_Edit /* 79 */,
-   XCF_Undo /* 80 */,		XCF_Redo /* 81 */,
-   XCF_Select_Save /* 82 */,	XCF_Unselect /* 83 */,
-   XCF_Dashed /* 84 */,		XCF_Dotted /* 85 */,
-   XCF_Solid /* 86 */,		XCF_Prompt /* 87 */,
-   XCF_Dot /* 88 */,		XCF_Wire /* 89 */,
-   XCF_Start /* 90 */,		XCF_Finish /* 91 */,
-   XCF_Cancel /* 92 */,		XCF_Nothing /* 93 */,
-   XCF_Exit /* 94 */,		XCF_Netlist /* 95 */,
-   XCF_Swap /* 96 */,		XCF_Pin_Label /* 97 */,
-   XCF_Pin_Global /* 98 */,	XCF_Info_Label /* 99 */,
-   XCF_Graphic	/* 100 */,	XCF_Connectivity /* 101 */,
-   XCF_Sim /* 102 */,		XCF_SPICE /* 103 */,
-   XCF_PCB /* 104 */,		XCF_SPICEflat /* 105 */,
+   XCF_Write /* 56 */,		XCF_Rotate /* 57 */,
+   XCF_Flip_X /* 58 */,		XCF_Flip_Y /* 59 */,
+   XCF_Snap /* 60 */,		XCF_SnapTo /* 61 */,
+   XCF_Pop /* 62 */,		XCF_Push /* 63 */,
+   XCF_Delete /* 64 */,		XCF_Select /* 65 */,
+   XCF_Box /* 66 */,		XCF_Arc /* 67 */,
+   XCF_Text /* 68 */,		XCF_Exchange /* 69 */,
+   XCF_Copy /* 70 */,		XCF_Move /* 71 */,
+   XCF_Join /* 72 */,		XCF_Unjoin /* 73 */,
+   XCF_Spline /* 74 */,		XCF_Edit /* 75 */,
+   XCF_Undo /* 76 */,		XCF_Redo /* 77 */,
+   XCF_Select_Save /* 78 */,	XCF_Unselect /* 79 */,
+   XCF_Dashed /* 80 */,		XCF_Dotted /* 81 */,
+   XCF_Solid /* 82 */,		XCF_Prompt /* 83 */,
+   XCF_Dot /* 84 */,		XCF_Wire /* 85 */,
+   XCF_Cancel /* 86 */,		XCF_Nothing /* 87 */,
+   XCF_Exit /* 88 */,		XCF_Netlist /* 89 */,
+   XCF_Swap /* 90 */,		XCF_Pin_Label /* 91 */,
+   XCF_Pin_Global /* 92 */,	XCF_Info_Label /* 93 */,
+   XCF_Graphic	/* 94 */,	XCF_Rescale /* 95 */,
+   XCF_SelectBox /* 96 */,	XCF_Connectivity /* 97 */,
+   XCF_Continue_Copy /* 98 */,	XCF_Continue_Element /* 99 */,
+   XCF_Finish_Copy /* 100 */,	XCF_Finish_Element /* 101 */,
+   XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
+   XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
+   XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
+   XCF_Text_Delete_Param /* 108 */,
    NUM_FUNCTIONS
 };
 
@@ -1282,10 +1312,11 @@ typedef struct {
    objectptr	editstack;
    Matrixptr	MatStack;
    pushlistptr	stack;
-   pushlistptr	drawstack;
+   pushlistptr	hierstack;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
+   Cursor	*defaultcursor;
 } Clientdata;
 
 /* Record for undo function */ 
@@ -1310,9 +1341,10 @@ typedef struct _undostack {
 /* the library page, and not be destroyed on every call to "composelib"	*/
 
 typedef struct {
+   char		*filename;	/* Last file loaded into the library */
    short	number;
    objectptr	*library;
-   liblistptr	instlist;  /* List of instances */
+   liblistptr	instlist;	/* List of instances */
 } Library;
 
 /*----------------------------------------------------------------------*/
@@ -1322,8 +1354,10 @@ typedef struct {
 typedef struct {
    char		*tempfile;
    char		*tempdir;
+   Boolean	retain_backup;
    xcIntervalId	timeout_id;
    int		save_interval;
+   short	do_replace;
    u_short	new_changes;
    short	numlibs;
    short	pages;
@@ -1360,6 +1394,8 @@ typedef struct _menustruct {
 /* Structure for calling routines from the Toolbar icons		*/
 /*----------------------------------------------------------------------*/
 
+typedef struct _toolbarstruct *toolbarptr;
+
 typedef struct _toolbarstruct {
 	char *name;
 	char **icon_data;
