@@ -27,6 +27,7 @@ extern int xctcl_standardaction();
 extern int xctcl_refresh();
 extern int xctcl_library();
 extern int xctcl_font();
+extern int xctcl_cursor();
 extern int xctcl_bind();
 extern int xctcl_filerecover();
 extern int xctcl_color();
@@ -50,7 +51,7 @@ extern int xctcl_push();
 extern int xctcl_pop();
 extern int xctcl_rotate();
 extern int xctcl_flip();
-extern int xctcl_set();
+extern int xctcl_config();
 extern int xctcl_page();
 extern int xctcl_spice();
 extern int xctcl_zoom();
@@ -83,6 +84,7 @@ static cmdstruct xc_commands[] =
    {"refresh", (void *)xctcl_refresh},
    {"library", (void *)xctcl_library},
    {"loadfont", (void *)xctcl_font},
+   {"cursor", (void *)xctcl_cursor},
    {"bindkey", (void *)xctcl_bind},
    {"filerecover", (void *)xctcl_filerecover},
    {"color", (void *)xctcl_color},
@@ -106,7 +108,7 @@ static cmdstruct xc_commands[] =
    {"pop", (void *)xctcl_pop},
    {"rotate", (void *)xctcl_rotate},
    {"flip", (void *)xctcl_flip},
-   {"config", (void *)xctcl_set},
+   {"config", (void *)xctcl_config},
    {"page", (void *)xctcl_page},
    {"zoom", (void *)xctcl_zoom},
    {"pan", (void *)xctcl_pan},
@@ -209,11 +211,6 @@ Xcircuit_Init(Tcl_Interp *interp)
    sprintf(version_string, "%g", PROG_VERSION);
    Tcl_SetVar(interp, "XCIRCUIT_VERSION", version_string, TCL_GLOBAL_ONLY);
 
-#ifdef ASG
-   /* Set a variable in Tcl indicating that the ASG module is compiled in */
-   Tcl_SetVar(interp, "XCIRCUIT_ASG", "1", TCL_GLOBAL_ONLY);
-#endif
-
    /* Export the namespace commands */
 
    Tcl_Eval(interp, "namespace eval xcircuit namespace export *");
